@@ -34,10 +34,10 @@ G_BEGIN_DECLS
 #define GST_DVDEC(obj) \
   (G_TYPE_CHECK_INSTANCE_CAST((obj),GST_TYPE_DVDEC,GstDVDec))
 #define GST_DVDEC_CLASS(klass) \
-  (G_TYPE_CHECK_CLASS_CAST((klass),GST_TYPE_DVDEC,GstDVDec))
+  (G_TYPE_CHECK_CLASS_CAST((klass),GST_TYPE_DVDEC,GstDVDecClass))
 #define GST_IS_DVDEC(obj) \
   (G_TYPE_CHECK_INSTANCE_TYPE((obj),GST_TYPE_DVDEC))
-#define GST_IS_DVDEC_CLASS(obj) \
+#define GST_IS_DVDEC_CLASS(klass) \
   (G_TYPE_CHECK_CLASS_TYPE((klass),GST_TYPE_DVDEC))
 
 
@@ -56,19 +56,28 @@ struct _GstDVDec {
   gboolean       clamp_chroma;
   gint           quality;
 
-  gint           frame_len;
   gboolean       PAL;
+  gboolean       wide;
+  gint           frame_len;
+
+  /* input caps */
+  gboolean       sink_negotiated;
   gint           framerate_numerator;
   gint           framerate_denominator;
   gint           height;
-  gboolean       wide;
+  gint           par_x;
+  gint           par_y;
+  gboolean       need_par;
 
   /* negotiated output */
   dv_color_space_t space;
   gint           bpp;
+  gboolean       src_negotiated;
   
   gint           video_offset;
   gint           drop_factor;
+
+  GstSegment	*segment;
 };
 
 struct _GstDVDecClass {
