@@ -17,12 +17,6 @@
  * Boston, MA 02111-1307, USA.
  */
 
-/**
- * SECTION:element-jpegdec
- *
- * Decodes jpeg images. If there is no framerate set on sink caps, it sends EOS
- * after the first picture.
- */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -993,8 +987,9 @@ gst_jpeg_dec_chain (GstPad * pad, GstBuffer * buf)
             GST_TIME_ARGS (clip_stop - clip_start));
         GST_BUFFER_DURATION (outbuf) = clip_stop - clip_start;
       }
-    } else
-      goto drop_buffer;
+    } else {
+      GST_WARNING_OBJECT (dec, "Outgoing buffer is outside configured segment");
+    }
   }
 
   GST_LOG_OBJECT (dec, "pushing buffer (ts=%" GST_TIME_FORMAT ", dur=%"
@@ -1060,13 +1055,6 @@ alloc_failed:
     }
     goto exit;
   }
-drop_buffer:
-  {
-    GST_WARNING_OBJECT (dec, "Outgoing buffer is outside configured segment");
-    gst_buffer_unref (outbuf);
-    ret = GST_FLOW_OK;
-    goto exit;
-  }
 }
 
 static gboolean
