@@ -23,7 +23,7 @@
 
 
 #include <setjmp.h>
-#include <gst/gstelement.h>
+#include <gst/gst.h>
 
 /* this is a hack hack hack to get around jpeglib header bugs... */
 #ifdef HAVE_STDLIB_H
@@ -38,10 +38,10 @@ G_BEGIN_DECLS
 #define GST_JPEG_DEC(obj) \
   (G_TYPE_CHECK_INSTANCE_CAST((obj),GST_TYPE_JPEG_DEC,GstJpegDec))
 #define GST_JPEG_DEC_CLASS(klass) \
-  (G_TYPE_CHECK_CLASS_CAST((klass),GST_TYPE_JPEG_DEC,GstJpegDec))
+  (G_TYPE_CHECK_CLASS_CAST((klass),GST_TYPE_JPEG_DEC,GstJpegDecClass))
 #define GST_IS_JPEG_DEC(obj) \
   (G_TYPE_CHECK_INSTANCE_TYPE((obj),GST_TYPE_JPEG_DEC))
-#define GST_IS_JPEG_DEC_CLASS(obj) \
+#define GST_IS_JPEG_DEC_CLASS(klass) \
   (G_TYPE_CHECK_CLASS_TYPE((klass),GST_TYPE_JPEG_DEC))
 
 typedef struct _GstJpegDec           GstJpegDec;
@@ -72,19 +72,27 @@ struct _GstJpegDec {
   /* TRUE if each input buffer contains a whole jpeg image */
   gboolean packetized;
 
-  /* the timestamp of the next frame */
+  /* the (expected) timestamp of the next frame */
   guint64  next_ts;
 
+  GstSegment *segment;
+
   /* video state */
   gint framerate_numerator;
   gint framerate_denominator;
-  gint frames_decoded;
 
   /* negotiated state */
   gint     caps_framerate_numerator;
   gint     caps_framerate_denominator;
   gint     caps_width;
   gint     caps_height;
+  gint     outsize;
+  /* temp space for samples */
+  gboolean direct;
+  guchar **scanarray[3];
+
+  /* properties */
+  gint     idct_method;
 
   struct jpeg_decompress_struct cinfo;
   struct GstJpegDecErrorMgr     jerr;
