@@ -26,6 +26,9 @@
 #include <gst/base/gstpushsrc.h>
 
 #include <libraw1394/raw1394.h>
+#ifdef HAVE_LIBIEC61883
+#include <libiec61883/iec61883.h>
+#endif
 
 G_BEGIN_DECLS
 
@@ -34,10 +37,10 @@ G_BEGIN_DECLS
 #define GST_DV1394SRC(obj) \
   (G_TYPE_CHECK_INSTANCE_CAST((obj),GST_TYPE_DV1394SRC,GstDV1394Src))
 #define GST_DV1394SRC_CLASS(klass) \
-  (G_TYPE_CHECK_CLASS_CAST((klass),GST_TYPE_DV1394SRC,GstDV1394Src))
+  (G_TYPE_CHECK_CLASS_CAST((klass),GST_TYPE_DV1394SRC,GstDV1394SrcClass))
 #define GST_IS_DV1394SRC(obj) \
   (G_TYPE_CHECK_INSTANCE_TYPE((obj),GST_TYPE_DV1394SRC))
-#define GST_IS_DV1394SRC_CLASS(obj) \
+#define GST_IS_DV1394SRC_CLASS(klass) \
   (G_TYPE_CHECK_CLASS_TYPE((klass),GST_TYPE_DV1394SRC))
 
 typedef struct _GstDV1394Src GstDV1394Src;
@@ -74,6 +77,9 @@ struct _GstDV1394Src {
   gchar *uri;
 
   gboolean connected;
+  #ifdef HAVE_LIBIEC61883
+  iec61883_dv_fb_t iec61883dv;
+  #endif
 };
 
 struct _GstDV1394SrcClass {
