@@ -23,9 +23,14 @@
 #include <gst/gst.h>
 #include <gst/video/video.h>
 
+#include <stdlib.h>
 #include <string.h>
 #include <math.h>
 
+#ifndef M_PI
+#define M_PI  3.14159265358979323846
+#endif
+
 #define GST_TYPE_ALPHA \
   (gst_alpha_get_type())
 #define GST_ALPHA(obj) \
@@ -34,7 +39,7 @@
   (G_TYPE_CHECK_CLASS_CAST((klass),GST_TYPE_ALPHA,GstAlphaClass))
 #define GST_IS_ALPHA(obj) \
   (G_TYPE_CHECK_INSTANCE_TYPE((obj),GST_TYPE_ALPHA))
-#define GST_IS_ALPHA_CLASS(obj) \
+#define GST_IS_ALPHA_CLASS(klass) \
   (G_TYPE_CHECK_CLASS_TYPE((klass),GST_TYPE_ALPHA))
 
 typedef struct _GstAlpha GstAlpha;
@@ -94,8 +99,8 @@ struct _GstAlphaClass
 };
 
 /* elementfactory information */
-static GstElementDetails gst_alpha_details =
-GST_ELEMENT_DETAILS ("alpha filter",
+static const GstElementDetails gst_alpha_details =
+GST_ELEMENT_DETAILS ("Alpha filter",
     "Filter/Effect/Video",
     "Adds an alpha channel to video",
     "Wim Taymans <wim@fluendo.com>");
@@ -170,7 +175,7 @@ static GType
 gst_alpha_method_get_type (void)
 {
   static GType alpha_method_type = 0;
-  static GEnumValue alpha_method[] = {
+  static const GEnumValue alpha_method[] = {
     {ALPHA_METHOD_SET, "Set/adjust alpha channel", "set"},
     {ALPHA_METHOD_GREEN, "Chroma Key green", "green"},
     {ALPHA_METHOD_BLUE, "Chroma Key blue", "blue"},
@@ -231,7 +236,7 @@ gst_alpha_class_init (GstAlphaClass * klass)
   gobject_class = (GObjectClass *) klass;
   gstelement_class = (GstElementClass *) klass;
 
-  parent_class = g_type_class_ref (GST_TYPE_ELEMENT);
+  parent_class = g_type_class_peek_parent (klass);
 
   gobject_class->set_property = gst_alpha_set_property;
   gobject_class->get_property = gst_alpha_get_property;
@@ -267,15 +272,13 @@ gst_alpha_init (GstAlpha * alpha)
 {
   /* create the sink and src pads */
   alpha->sinkpad =
-      gst_pad_new_from_template (gst_static_pad_template_get
-      (&gst_alpha_sink_template), "sink");
+      gst_pad_new_from_static_template (&gst_alpha_sink_template, "sink");
   gst_element_add_pad (GST_ELEMENT (alpha), alpha->sinkpad);
   gst_pad_set_chain_function (alpha->sinkpad, gst_alpha_chain);
   gst_pad_set_setcaps_function (alpha->sinkpad, gst_alpha_sink_setcaps);
 
   alpha->srcpad =
-      gst_pad_new_from_template (gst_static_pad_template_get
-      (&gst_alpha_src_template), "src");
+      gst_pad_new_from_static_template (&gst_alpha_src_template, "src");
   gst_element_add_pad (GST_ELEMENT (alpha), alpha->srcpad);
 
   alpha->alpha = DEFAULT_ALPHA;
@@ -857,6 +860,7 @@ gst_alpha_chain (GstPad * pad, GstBuffer * buffer)
 static GstStateChangeReturn
 gst_alpha_change_state (GstElement * element, GstStateChange transition)
 {
+  GstStateChangeReturn res;
   GstAlpha *alpha;
 
   alpha = GST_ALPHA (element);
@@ -868,18 +872,21 @@ gst_alpha_change_state (GstElement * element, GstStateChange transition)
       gst_alpha_init_params (alpha);
       break;
     case GST_STATE_CHANGE_PAUSED_TO_PLAYING:
-      break;
-    case GST_STATE_CHANGE_PLAYING_TO_PAUSED:
-      break;
-    case GST_STATE_CHANGE_PAUSED_TO_READY:
-      break;
-    case GST_STATE_CHANGE_READY_TO_NULL:
+    default:
       break;
   }
 
-  parent_class->change_state (element, transition);
+  res = parent_class->change_state (element, transition);
 
-  return GST_STATE_CHANGE_SUCCESS;
+  switch (transition) {
+    case GST_STATE_CHANGE_PLAYING_TO_PAUSED:
+    case GST_STATE_CHANGE_PAUSED_TO_READY:
+    case GST_STATE_CHANGE_READY_TO_NULL:
+    default:
+      break;
+  }
+
+  return res;
 }
 
 static gboolean
