@@ -59,7 +59,7 @@
   (G_TYPE_CHECK_CLASS_CAST((klass),GST_TYPE_REVTV,GstRevTVClass))
 #define GST_IS_REVTV(obj) \
   (G_TYPE_CHECK_INSTANCE_TYPE((obj),GST_TYPE_REVTV))
-#define GST_IS_REVTV_CLASS(obj) \
+#define GST_IS_REVTV_CLASS(klass) \
   (G_TYPE_CHECK_CLASS_TYPE((klass),GST_TYPE_REVTV))
 
 #define THE_COLOR 0xffffffff
