@@ -1,5 +1,6 @@
 /* -*- Mode: C; tab-width: 2; indent-tabs-mode: t; c-basic-offset: 2 -*- */
-/* Copyright 2005 Jan Schmidt <thaytan@mad.scientist.com>
+/* Copyright 2006 Tim-Philipp Müller <tim centricular net>
+ * Copyright 2005 Jan Schmidt <thaytan@mad.scientist.com>
  * Copyright 2002,2003 Scott Wheeler <wheeler@kde.org> (portions from taglib)
  *
  * This library is free software; you can redistribute it and/or
@@ -25,6 +26,7 @@
 #include <string.h>
 #include <stdlib.h>
 #include <gst/tag/tag.h>
+#include <gst/base/gsttypefindhelper.h>
 
 #ifdef HAVE_ZLIB
 #include <zlib.h>
@@ -35,13 +37,14 @@
 GST_DEBUG_CATEGORY_EXTERN (id3demux_debug);
 #define GST_CAT_DEFAULT (id3demux_debug)
 
-static gchar *parse_comment_frame (ID3TagsWorking * work);
+static gboolean parse_comment_frame (ID3TagsWorking * work);
 static GArray *parse_text_identification_frame (ID3TagsWorking * work);
 static gchar *parse_user_text_identification_frame (ID3TagsWorking * work,
     const gchar ** tag_name);
 static gchar *parse_unique_file_identifier (ID3TagsWorking * work,
     const gchar ** tag_name);
 static gboolean parse_relative_volume_adjustment_two (ID3TagsWorking * work);
+static void parse_obsolete_tdat_frame (ID3TagsWorking * work);
 static gboolean id3v2_tag_to_taglist (ID3TagsWorking * work,
     const gchar * tag_name, const gchar * tag_str);
 /* Parse a single string into an array of gchar* */
@@ -51,6 +54,7 @@ static void free_tag_strings (GArray * fields);
 static gboolean
 id3v2_genre_fields_to_taglist (ID3TagsWorking * work, const gchar * tag_name,
     GArray * tag_fields);
+static gboolean parse_picture_frame (ID3TagsWorking * work);
 
 #define ID3V2_ENCODING_ISO8859 0x00
 #define ID3V2_ENCODING_UTF16   0x01
@@ -70,6 +74,10 @@ id3demux_id3v2_parse_frame (ID3TagsWorking * work)
   gchar *tag_str = NULL;
   GArray *tag_fields = NULL;
 
+#ifdef HAVE_ZLIB
+  guint8 *uncompressed_data = NULL;
+#endif
+
   /* Check that the frame id is valid */
   for (i = 0; i < 5 && work->frame_id[i] != '\0'; i++) {
     if (!g_ascii_isalnum (work->frame_id[i])) {
@@ -93,6 +101,7 @@ id3demux_id3v2_parse_frame (ID3TagsWorking * work)
   if (tag_name == NULL &&
       strncmp (work->frame_id, "RVA2", 4) != 0 &&
       strncmp (work->frame_id, "TXXX", 4) != 0 &&
+      strncmp (work->frame_id, "TDAT", 4) != 0 &&
       strncmp (work->frame_id, "UFID", 4) != 0) {
     return FALSE;
   }
@@ -118,22 +127,23 @@ id3demux_id3v2_parse_frame (ID3TagsWorking * work)
     uLongf destSize = work->parse_size;
     Bytef *dest, *src;
 
-    work->parse_data = g_malloc (work->parse_size);
-    g_return_val_if_fail (work->parse_data != NULL, FALSE);
+    uncompressed_data = g_malloc (work->parse_size);
 
-    dest = (Bytef *) work->parse_data;
+    dest = (Bytef *) uncompressed_data;
     src = (Bytef *) frame_data;
 
     if (uncompress (dest, &destSize, src, frame_data_size) != Z_OK) {
-      g_free (work->parse_data);
+      g_free (uncompressed_data);
       return FALSE;
     }
     if (destSize != work->parse_size) {
       GST_WARNING
-          ("Decompressing ID3v2 frame %s did not produce expected size %d bytes (got %d)",
-          tag_name, work->parse_data, destSize);
+          ("Decompressing ID3v2 frame %s did not produce expected size %d bytes (got %lu)",
+          tag_name, work->parse_size, destSize);
+      g_free (uncompressed_data);
       return FALSE;
     }
+    work->parse_data = uncompressed_data;
 #else
     GST_WARNING ("Compressed ID3v2 tag frame could not be decompressed"
         " because gstid3demux was compiled without zlib support");
@@ -144,18 +154,22 @@ id3demux_id3v2_parse_frame (ID3TagsWorking * work)
   }
 
   if (work->frame_id[0] == 'T') {
-    if (strcmp (work->frame_id, "TXXX") != 0) {
-      /* Text identification frame */
-      tag_fields = parse_text_identification_frame (work);
-    } else {
+    if (strcmp (work->frame_id, "TDAT") == 0) {
+      parse_obsolete_tdat_frame (work);
+      result = TRUE;
+    } else if (strcmp (work->frame_id, "TXXX") == 0) {
       /* Handle user text frame */
       tag_str = parse_user_text_identification_frame (work, &tag_name);
+    } else {
+      /* Text identification frame */
+      tag_fields = parse_text_identification_frame (work);
     }
   } else if (!strcmp (work->frame_id, "COMM")) {
     /* Comment */
-    tag_str = parse_comment_frame (work);
+    result = parse_comment_frame (work);
   } else if (!strcmp (work->frame_id, "APIC")) {
     /* Attached picture */
+    result = parse_picture_frame (work);
   } else if (!strcmp (work->frame_id, "RVA2")) {
     /* Relative volume */
     result = parse_relative_volume_adjustment_two (work);
@@ -163,9 +177,13 @@ id3demux_id3v2_parse_frame (ID3TagsWorking * work)
     /* Unique file identifier */
     tag_str = parse_unique_file_identifier (work, &tag_name);
   }
-
-  if (work->frame_flags & ID3V2_FRAME_FORMAT_COMPRESSION)
-    g_free (work->parse_data);
+#ifdef HAVE_ZLIB
+  if (work->frame_flags & ID3V2_FRAME_FORMAT_COMPRESSION) {
+    g_free (uncompressed_data);
+    uncompressed_data = NULL;
+    work->parse_data = frame_data;
+  }
+#endif
 
   if (tag_str != NULL) {
     /* g_print ("Tag %s value %s\n", tag_name, tag_str); */
@@ -191,17 +209,17 @@ id3demux_id3v2_parse_frame (ID3TagsWorking * work)
   return result;
 }
 
-static gchar *
+static gboolean
 parse_comment_frame (ID3TagsWorking * work)
 {
+  guint dummy;
   guint8 encoding;
   gchar language[4];
   GArray *fields = NULL;
-  gchar *out_str = NULL;
   gchar *description, *text;
 
   if (work->parse_size < 6)
-    return NULL;
+    return FALSE;
 
   encoding = work->parse_data[0];
   language[0] = work->parse_data[1];
@@ -222,19 +240,37 @@ parse_comment_frame (ID3TagsWorking * work)
   if (!g_utf8_validate (text, -1, NULL)) {
     GST_WARNING ("Converted string is not valid utf-8");
     goto fail;
-  } else {
-    if (strlen (description) > 0 && g_utf8_validate (description, -1, NULL)) {
-      out_str = g_strdup_printf ("Description: %s\nComment: %s",
-          description, text);
-    } else {
-      out_str = g_strdup (text);
-    }
   }
 
-fail:
-  free_tag_strings (fields);
+  /* skip our own dummy descriptions (from id3v2mux) */
+  if (strlen (description) > 0 && g_utf8_validate (description, -1, NULL) &&
+      sscanf (description, "c%u", &dummy) != 1) {
+    gchar *s;
 
-  return out_str;
+    if (language[0] != '\0') {
+      s = g_strdup_printf ("%s[%s]=%s", description, language, text);
+    } else {
+      s = g_strdup_printf ("%s=%s", description, text);
+    }
+    gst_tag_list_add (work->tags, GST_TAG_MERGE_APPEND,
+        GST_TAG_EXTENDED_COMMENT, s, NULL);
+    g_free (s);
+  } else if (text != NULL && *text != '\0') {
+    gst_tag_list_add (work->tags, GST_TAG_MERGE_APPEND,
+        GST_TAG_COMMENT, text, NULL);
+  } else {
+    goto fail;
+  }
+
+  free_tag_strings (fields);
+  return TRUE;
+
+fail:
+  {
+    GST_WARNING ("failed to parse COMM frame");
+    free_tag_strings (fields);
+    return FALSE;
+  }
 }
 
 static GArray *
@@ -251,12 +287,12 @@ parse_text_identification_frame (ID3TagsWorking * work)
       work->parse_size - 1, &fields);
   if (fields) {
     if (fields->len > 0) {
-      GST_LOG ("Read %d fields from Text ID frame of size %d. First is '%s'",
-          fields->len, work->parse_size - 1,
+      GST_LOG ("Read %d fields from Text ID frame of size %d with encoding %d"
+          ". First is '%s'", fields->len, work->parse_size - 1, encoding,
           g_array_index (fields, gchar *, 0));
     } else {
-      GST_LOG ("Read %d fields from Text ID frame of size %d", fields->len,
-          work->parse_size - 1);
+      GST_LOG ("Read 0 fields from Text ID frame of size %d with encoding %d",
+          work->parse_size - 1, encoding);
     }
   }
 
@@ -359,6 +395,198 @@ parse_unique_file_identifier (ID3TagsWorking * work, const gchar ** tag_name)
   return ret;
 }
 
+/* parse data and return length of the next string in the given encoding,
+ * including the NUL terminator */
+static gint
+scan_encoded_string (guint8 encoding, gchar * data, gint data_size)
+{
+  gint i;
+
+  switch (encoding) {
+    case ID3V2_ENCODING_ISO8859:
+    case ID3V2_ENCODING_UTF8:
+      for (i = 0; i < data_size; ++i) {
+        if (data[i] == '\0')
+          return i + 1;
+      }
+      break;
+    case ID3V2_ENCODING_UTF16:
+    case ID3V2_ENCODING_UTF16BE:
+      /* we don't care about BOMs here and treat them as part of the string */
+      /* Find '\0\0' terminator */
+      for (i = 0; i < data_size - 1; i += 2) {
+        if (data[i] == '\0' && data[i + 1] == '\0')
+          return i + 2;
+      }
+      break;
+    default:
+      break;
+  }
+
+  return 0;
+}
+
+static gboolean
+parse_picture_frame (ID3TagsWorking * work)
+{
+  GstBuffer *image = NULL;
+  GstCaps *image_caps = NULL;
+  gboolean is_pic_uri = FALSE;
+  guint8 txt_encoding, pic_type;
+  gchar *c, *mime_str = NULL;
+  gint len, datalen;
+
+  GST_LOG ("APIC frame (ID3v2.%u)", ID3V2_VER_MAJOR (work->hdr.version));
+
+  if (work->parse_size < 1 + 1 + 1 + 1 + 1)
+    goto not_enough_data;
+
+  txt_encoding = work->parse_data[0];
+  ++work->parse_data;
+  --work->parse_size;
+
+  /* Read image format; in early ID3v2 versions this is a fixed-length
+   * 3-character string without terminator; in later versions (>= 2.3.0)
+   * this is a NUL-terminated string of variable length */
+  if (ID3V2_VER_MAJOR (work->hdr.version) < 3) {
+    if (work->parse_size < 3)
+      goto not_enough_data;
+
+    mime_str = g_strndup ((gchar *) work->parse_data, 3);
+    len = 3;
+  } else {
+    if (!parse_id_string (work, &mime_str, &len, &datalen))
+      return FALSE;
+    ++len;                      /* for string terminator */
+  }
+
+  /* Fix up 'jpg' => 'jpeg' in mime/media type */
+  if (mime_str != NULL &&
+      (g_ascii_strcasecmp (mime_str, "jpg") == 0 ||
+          g_ascii_strcasecmp (mime_str, "image/jpg") == 0)) {
+    g_free (mime_str);
+    mime_str = g_strdup ("jpeg");
+  }
+
+  /* Make lower-case */
+  for (c = mime_str; c != NULL && *c != '\0'; ++c) {
+    *c = g_ascii_tolower (*c);
+  }
+
+  is_pic_uri = (mime_str != NULL && strcmp (mime_str, "-->") == 0);
+
+  if (mime_str && *mime_str && strchr (mime_str, '/') == NULL && !is_pic_uri) {
+    gchar *tmp;
+
+    tmp = g_strdup_printf ("image/%s", mime_str);
+    g_free (mime_str);
+    mime_str = tmp;
+  }
+
+  if (work->parse_size < len + 1 + 1 + 1)
+    goto not_enough_data;
+
+  work->parse_data += len;
+  work->parse_size -= len;
+
+  /* Read image type */
+  pic_type = work->parse_data[0];
+  ++work->parse_data;
+  --work->parse_size;
+
+  GST_LOG ("APIC frame mime type    : %s", GST_STR_NULL (mime_str));
+  GST_LOG ("APIC frame picture type : 0x%02x", (guint) pic_type);
+
+  if (work->parse_size < 1 + 1)
+    goto not_enough_data;
+
+  len = scan_encoded_string (txt_encoding, (gchar *) work->parse_data,
+      work->parse_size);
+
+  if (len < 1)
+    goto error;
+
+  /* just skip the description string ... */
+  GST_LOG ("Skipping description string (%d bytes in original coding)", len);
+
+  if (work->parse_size < len + 1)
+    goto not_enough_data;
+
+  work->parse_data += len;
+  work->parse_size -= len;
+
+  GST_DEBUG ("image data is %u bytes", work->parse_size);
+
+  if (work->parse_size <= 0)
+    goto not_enough_data;
+
+  if (is_pic_uri) {
+    gchar *uri;
+
+    uri = g_strndup ((gchar *) work->parse_data, work->parse_size);
+    GST_DEBUG ("image URI: %s", uri);
+
+    image = gst_buffer_new ();
+    GST_BUFFER_MALLOCDATA (image) = (guint8 *) uri;     /* take ownership */
+    GST_BUFFER_DATA (image) = (guint8 *) uri;
+    GST_BUFFER_SIZE (image) = work->parse_size;
+
+    image_caps = gst_caps_new_simple ("text/uri-list", NULL);
+  } else {
+    image = gst_buffer_new_and_alloc (work->parse_size);
+    memcpy (GST_BUFFER_DATA (image), work->parse_data, work->parse_size);
+
+    /* if possible use GStreamer media type rather than declared type */
+    image_caps = gst_type_find_helper_for_buffer (NULL, image, NULL);
+    if (image_caps) {
+      GST_DEBUG ("Found GStreamer media type: %" GST_PTR_FORMAT, image_caps);
+    } else if (mime_str && *mime_str) {
+      GST_DEBUG ("No GStreamer media type found, using declared type");
+      image_caps = gst_caps_new_simple (mime_str, NULL);
+    } else {
+      GST_DEBUG ("Empty declared mime type, ignoring image frame");
+      image = NULL;
+      image_caps = NULL;
+      goto error;
+    }
+  }
+
+  if (image && image_caps) {
+    if (pic_type > 0x14)
+      pic_type = GST_TAG_IMAGE_TYPE_UNDEFINED;
+    gst_structure_set (gst_caps_get_structure (image_caps, 0),
+        "image-type", GST_TYPE_TAG_IMAGE_TYPE,
+        (GstTagImageType) pic_type, NULL);
+
+    gst_buffer_set_caps (image, image_caps);
+    gst_caps_unref (image_caps);
+    if (pic_type == 0x01 || pic_type == 0x02) {
+      /* file icon of some sort */
+      gst_tag_list_add (work->tags, GST_TAG_MERGE_APPEND,
+          GST_TAG_PREVIEW_IMAGE, image, NULL);
+    } else {
+      gst_tag_list_add (work->tags, GST_TAG_MERGE_APPEND,
+          GST_TAG_IMAGE, image, NULL);
+    }
+    gst_buffer_unref (image);
+  }
+
+  g_free (mime_str);
+  return TRUE;
+
+not_enough_data:
+  {
+    GST_DEBUG ("not enough data, skipping APIC frame");
+    /* fall through to error */
+  }
+error:
+  {
+    GST_DEBUG ("problem parsing APIC frame, skipping");
+    g_free (mime_str);
+    return FALSE;
+  }
+}
+
 #define ID3V2_RVA2_CHANNEL_MASTER  1
 
 static gboolean
@@ -437,6 +665,23 @@ parse_relative_volume_adjustment_two (ID3TagsWorking * work)
   return (gain_tag_name != NULL || peak_tag_name != NULL);
 }
 
+static void
+parse_obsolete_tdat_frame (ID3TagsWorking * work)
+{
+  if (work->parse_size >= 5 &&
+      work->parse_data[0] == ID3V2_ENCODING_ISO8859 &&
+      g_ascii_isdigit (work->parse_data[1]) &&
+      g_ascii_isdigit (work->parse_data[2]) &&
+      g_ascii_isdigit (work->parse_data[3]) &&
+      g_ascii_isdigit (work->parse_data[4])) {
+    work->pending_day = (10 * g_ascii_digit_value (work->parse_data[1])) +
+        g_ascii_digit_value (work->parse_data[2]);
+    work->pending_month = (10 * g_ascii_digit_value (work->parse_data[3])) +
+        g_ascii_digit_value (work->parse_data[4]);
+    GST_LOG ("date (dd/mm) %02u/%02u", work->pending_day, work->pending_month);
+  }
+}
+
 static gboolean
 id3v2_tag_to_taglist (ID3TagsWorking * work, const gchar * tag_name,
     const gchar * tag_str)
@@ -450,41 +695,34 @@ id3v2_tag_to_taglist (ID3TagsWorking * work, const gchar * tag_name,
   switch (tag_type) {
     case G_TYPE_UINT:
     {
-      guint tmp;
-      gchar *check;
+      gint current, total;
 
-      tmp = strtoul ((char *) tag_str, &check, 10);
-
-      if (strcmp (tag_name, GST_TAG_TRACK_NUMBER) == 0) {
-        if (*check == '/') {
-          guint total;
-
-          check++;
-          total = strtoul (check, &check, 10);
-          if (*check != '\0')
-            break;
-
-          gst_tag_list_add (tag_list, GST_TAG_MERGE_APPEND,
-              GST_TAG_TRACK_COUNT, total, NULL);
-        }
-      } else if (strcmp (tag_name, GST_TAG_ALBUM_VOLUME_NUMBER) == 0) {
-        if (*check == '/') {
-          guint total;
-
-          check++;
-          total = strtoul (check, &check, 10);
-          if (*check != '\0')
-            break;
-
-          gst_tag_list_add (tag_list, GST_TAG_MERGE_APPEND,
-              GST_TAG_ALBUM_VOLUME_COUNT, total, NULL);
+      if (sscanf (tag_str, "%d/%d", &current, &total) == 2) {
+        if (total <= 0) {
+          GST_WARNING ("Ignoring invalid value for total %d in tag %s",
+              total, tag_name);
+        } else {
+          if (strcmp (tag_name, GST_TAG_TRACK_NUMBER) == 0) {
+            gst_tag_list_add (tag_list, GST_TAG_MERGE_APPEND,
+                GST_TAG_TRACK_COUNT, total, NULL);
+          } else if (strcmp (tag_name, GST_TAG_ALBUM_VOLUME_NUMBER) == 0) {
+            gst_tag_list_add (tag_list, GST_TAG_MERGE_APPEND,
+                GST_TAG_ALBUM_VOLUME_COUNT, total, NULL);
+          }
         }
+      } else if (sscanf (tag_str, "%d", &current) != 1) {
+        /* Not an integer in the string */
+        GST_WARNING ("Tag string for tag %s does not contain an integer - "
+            "ignoring", tag_name);
+        break;
       }
 
-      if (*check != '\0')
-        break;
-
-      gst_tag_list_add (tag_list, GST_TAG_MERGE_APPEND, tag_name, tmp, NULL);
+      if (current <= 0) {
+        GST_WARNING ("Ignoring invalid value %d in tag %s", current, tag_name);
+      } else {
+        gst_tag_list_add (tag_list, GST_TAG_MERGE_APPEND, tag_name, current,
+            NULL);
+      }
       break;
     }
     case G_TYPE_UINT64:
@@ -622,8 +860,17 @@ id3v2_genre_fields_to_taglist (ID3TagsWorking * work, const gchar * tag_name,
         gint pos;
         gboolean found = FALSE;
 
-        /* Double parenthesis ends the numeric genres */
-        if (tag_str[0] == '(' && tag_str[1] == '(')
+        /* Double parenthesis ends the numeric genres, but we need
+         * to swallow the first one so we actually output '(' */
+        if (tag_str[0] == '(' && tag_str[1] == '(') {
+          tag_str++;
+          len--;
+          break;
+        }
+
+        /* If the first char is not a parenthesis, then stop
+         * looking for parenthesised genre strings */
+        if (tag_str[0] != '(')
           break;
 
         for (pos = 1; pos < len; pos++) {
@@ -640,6 +887,12 @@ id3v2_genre_fields_to_taglist (ID3TagsWorking * work, const gchar * tag_name,
             found = TRUE;
             break;
           }
+
+          /* If we encounter a non-digit while searching for a closing 
+           * parenthesis, we should not try and interpret this as a 
+           * numeric genre string */
+          if (!g_ascii_isdigit (tag_str[pos]))
+            break;
         }
         if (!found)
           break;                /* There was no closing parenthesis */
@@ -652,33 +905,21 @@ id3v2_genre_fields_to_taglist (ID3TagsWorking * work, const gchar * tag_name,
   return result;
 }
 
-static void
-parse_insert_string_field (const gchar * encoding, gchar * data, gint data_size,
-    GArray * fields)
-{
-  gchar *field;
-
-  field = g_convert (data, data_size, "UTF-8", encoding, NULL, NULL, NULL);
-  if (field && !g_utf8_validate (field, -1, NULL)) {
-    GST_DEBUG ("%s was bad UTF-8. Ignoring", field);
-    g_free (field);
-    field = NULL;
-  }
-  if (field)
-    g_array_append_val (fields, field);
-}
+static const gchar utf16enc[] = "UTF-16";
+static const gchar utf16leenc[] = "UTF-16LE";
+static const gchar utf16beenc[] = "UTF-16BE";
 
 static gboolean
-has_utf16_bom (gchar * data, const gchar ** p_in_encoding)
+find_utf16_bom (gchar * data, const gchar ** p_in_encoding)
 {
   guint16 marker = (GST_READ_UINT8 (data) << 8) | GST_READ_UINT8 (data + 1);
 
   switch (marker) {
     case 0xFFFE:
-      *p_in_encoding = "UTF16LE";
+      *p_in_encoding = utf16leenc;
       return TRUE;
     case 0xFEFF:
-      *p_in_encoding = "UTF16BE";
+      *p_in_encoding = utf16beenc;
       return TRUE;
     default:
       break;
@@ -686,6 +927,66 @@ has_utf16_bom (gchar * data, const gchar ** p_in_encoding)
   return FALSE;
 }
 
+static void
+parse_insert_string_field (guint8 encoding, gchar * data, gint data_size,
+    GArray * fields)
+{
+  gchar *field = NULL;
+
+  switch (encoding) {
+    case ID3V2_ENCODING_UTF16:
+    case ID3V2_ENCODING_UTF16BE:
+    {
+      const gchar *in_encode;
+
+      if (encoding == ID3V2_ENCODING_UTF16)
+        in_encode = utf16enc;
+      else
+        in_encode = utf16beenc;
+
+      /* Sometimes we see strings with multiple BOM markers at the start.
+       * In that case, we assume the innermost one is correct. If that fails
+       * to produce valid UTF-8, we try the other endianness anyway */
+      while (data_size > 2 && find_utf16_bom (data, &in_encode)) {
+        data += 2;              /* skip BOM */
+        data_size -= 2;
+      }
+
+      field = g_convert (data, data_size, "UTF-8", in_encode, NULL, NULL, NULL);
+
+      if (field == NULL || g_utf8_validate (field, -1, NULL) == FALSE) {
+        /* As a fallback, try interpreting UTF-16 in the other endianness */
+        if (in_encode == utf16beenc)
+          field = g_convert (data, data_size, "UTF-8", utf16leenc,
+              NULL, NULL, NULL);
+      }
+    }
+
+      break;
+    case ID3V2_ENCODING_ISO8859:
+      if (g_utf8_validate (data, data_size, NULL))
+        field = g_strndup (data, data_size);
+      else
+        field = g_convert (data, data_size, "UTF-8", "ISO-8859-1",
+            NULL, NULL, NULL);
+      break;
+    default:
+      field = g_strndup (data, data_size);
+      break;
+  }
+
+  if (field) {
+    if (g_utf8_validate (field, -1, NULL)) {
+      g_array_append_val (fields, field);
+      return;
+    }
+
+    GST_DEBUG ("%s was bad UTF-8 after conversion from encoding %d. Ignoring",
+        field, encoding);
+    g_free (field);
+  }
+}
+
 static void
 parse_split_strings (guint8 encoding, gchar * data, gint data_size,
     GArray ** out_fields)
@@ -700,13 +1001,13 @@ parse_split_strings (guint8 encoding, gchar * data, gint data_size,
     case ID3V2_ENCODING_ISO8859:
       for (text_pos = 0; text_pos < data_size; text_pos++) {
         if (data[text_pos] == 0) {
-          parse_insert_string_field ("ISO-8859-1", data + prev,
+          parse_insert_string_field (encoding, data + prev,
               text_pos - prev + 1, fields);
           prev = text_pos + 1;
         }
       }
       if (data_size - prev > 0 && data[prev] != 0x00) {
-        parse_insert_string_field ("ISO-8859-1", data + prev,
+        parse_insert_string_field (encoding, data + prev,
             data_size - prev, fields);
       }
 
@@ -714,34 +1015,24 @@ parse_split_strings (guint8 encoding, gchar * data, gint data_size,
     case ID3V2_ENCODING_UTF8:
       for (prev = 0, text_pos = 0; text_pos < data_size; text_pos++) {
         if (data[text_pos] == '\0') {
-          parse_insert_string_field ("UTF-8", data + prev,
+          parse_insert_string_field (encoding, data + prev,
               text_pos - prev + 1, fields);
           prev = text_pos + 1;
         }
       }
       if (data_size - prev > 0 && data[prev] != 0x00) {
-        parse_insert_string_field ("UTF-8", data + prev,
+        parse_insert_string_field (encoding, data + prev,
             data_size - prev, fields);
       }
       break;
     case ID3V2_ENCODING_UTF16:
     case ID3V2_ENCODING_UTF16BE:
     {
-      const gchar *in_encode;
-
-      if (encoding == ID3V2_ENCODING_UTF16)
-        in_encode = "UTF-16";
-      else
-        in_encode = "UTF-16BE";
-
       /* Find '\0\0' terminator */
       for (text_pos = 0; text_pos < data_size - 1; text_pos += 2) {
         if (data[text_pos] == '\0' && data[text_pos + 1] == '\0') {
-          if (has_utf16_bom (data + prev, &in_encode)) {
-            prev += 2;          /* skip BOM */
-          }
           /* found a delimiter */
-          parse_insert_string_field (in_encode, data + prev,
+          parse_insert_string_field (encoding, data + prev,
               text_pos - prev + 2, fields);
           text_pos++;           /* Advance to the 2nd NULL terminator */
           prev = text_pos + 1;
@@ -750,11 +1041,8 @@ parse_split_strings (guint8 encoding, gchar * data, gint data_size,
       }
       if (data_size - prev > 1 &&
           (data[prev] != 0x00 || data[prev + 1] != 0x00)) {
-        if (has_utf16_bom (data + prev, &in_encode)) {
-          prev += 2;            /* skip BOM */
-        }
         /* There were 2 or more non-null chars left, convert those too */
-        parse_insert_string_field (in_encode, data + prev,
+        parse_insert_string_field (encoding, data + prev,
             data_size - prev, fields);
       }
       break;
