@@ -108,6 +108,11 @@ static gboolean gst_rtp_xqt_depay_setcaps (GstBaseRTPDepayload * depayload,
 static GstBuffer *gst_rtp_xqt_depay_process (GstBaseRTPDepayload * depayload,
     GstBuffer * buf);
 
+static void gst_rtp_xqt_depay_set_property (GObject * object, guint prop_id,
+    const GValue * value, GParamSpec * pspec);
+static void gst_rtp_xqt_depay_get_property (GObject * object, guint prop_id,
+    GValue * value, GParamSpec * pspec);
+
 static GstStateChangeReturn gst_rtp_xqt_depay_change_state (GstElement *
     element, GstStateChange transition);
 
@@ -139,6 +144,9 @@ gst_rtp_xqt_depay_class_init (GstRtpXQTDepayClass * klass)
 
   gobject_class->finalize = gst_rtp_xqt_depay_finalize;
 
+  gobject_class->set_property = gst_rtp_xqt_depay_set_property;
+  gobject_class->get_property = gst_rtp_xqt_depay_get_property;
+
   gstelement_class->change_state = gst_rtp_xqt_depay_change_state;
 
   gstbasertpdepayload_class->set_caps = gst_rtp_xqt_depay_setcaps;
@@ -245,8 +253,11 @@ static gboolean
 gst_rtp_xqt_depay_setcaps (GstBaseRTPDepayload * depayload, GstCaps * caps)
 {
   GstStructure *structure;
+  GstRtpXQTDepay *rtpxqtdepay;
   gint clock_rate = 90000;      /* default */
 
+  rtpxqtdepay = GST_RTP_XQT_DEPAY (depayload);
+
   structure = gst_caps_get_structure (caps, 0);
 
   gst_structure_get_int (structure, "clock-rate", &clock_rate);
@@ -271,7 +282,6 @@ gst_rtp_xqt_depay_process (GstBaseRTPDepayload * depayload, GstBuffer * buf)
     /* discont, clear adapter and try to find a new packet start */
     gst_adapter_clear (rtpxqtdepay->adapter);
     rtpxqtdepay->need_resync = TRUE;
-    GST_DEBUG_OBJECT (rtpxqtdepay, "we need resync");
   }
 
   m = gst_rtp_buffer_get_marker (buf);
@@ -425,9 +435,7 @@ gst_rtp_xqt_depay_process (GstBaseRTPDepayload * depayload, GstBuffer * buf)
         switch (tlv_type) {
           case TLV_sd:
             /* Session description */
-            if (!gst_rtp_quicktime_parse_sd (rtpxqtdepay, payload, tlv_len))
-              goto unknown_format;
-            rtpxqtdepay->have_sd = TRUE;
+            gst_rtp_quicktime_parse_sd (rtpxqtdepay, payload, tlv_len);
             break;
           case TLV_qt:
           case TLV_ti:
@@ -657,11 +665,35 @@ wrong_length:
         ("Wrong payload length."), (NULL));
     return NULL;
   }
-unknown_format:
-  {
-    GST_ELEMENT_WARNING (rtpxqtdepay, STREAM, DECODE,
-        ("Unknown payload format."), (NULL));
-    return NULL;
+}
+
+static void
+gst_rtp_xqt_depay_set_property (GObject * object, guint prop_id,
+    const GValue * value, GParamSpec * pspec)
+{
+  GstRtpXQTDepay *rtpxqtdepay;
+
+  rtpxqtdepay = GST_RTP_XQT_DEPAY (object);
+
+  switch (prop_id) {
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+      break;
+  }
+}
+
+static void
+gst_rtp_xqt_depay_get_property (GObject * object, guint prop_id, GValue * value,
+    GParamSpec * pspec)
+{
+  GstRtpXQTDepay *rtpxqtdepay;
+
+  rtpxqtdepay = GST_RTP_XQT_DEPAY (object);
+
+  switch (prop_id) {
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+      break;
   }
 }
 
@@ -678,8 +710,7 @@ gst_rtp_xqt_depay_change_state (GstElement * element, GstStateChange transition)
       gst_adapter_clear (rtpxqtdepay->adapter);
       rtpxqtdepay->previous_id = -1;
       rtpxqtdepay->current_id = -1;
-      rtpxqtdepay->need_resync = TRUE;
-      rtpxqtdepay->have_sd = FALSE;
+      rtpxqtdepay->need_resync = FALSE;
       break;
     default:
       break;
