@@ -23,18 +23,12 @@
 
 #include "gstrtpdepay.h"
 #include "gstrtpac3depay.h"
-#include "gstrtpdvdepay.h"
-#include "gstrtpdvpay.h"
 #include "gstrtpilbcdepay.h"
 #include "gstrtpilbcpay.h"
 #include "gstrtppcmupay.h"
 #include "gstrtppcmapay.h"
 #include "gstrtppcmadepay.h"
 #include "gstrtppcmudepay.h"
-#include "gstrtpg726depay.h"
-#include "gstrtpg726pay.h"
-#include "gstrtpg729depay.h"
-#include "gstrtpg729pay.h"
 #include "gstrtpgsmpay.h"
 #include "gstrtpgsmdepay.h"
 #include "gstrtpamrpay.h"
@@ -42,25 +36,19 @@
 #include "gstrtpmpapay.h"
 #include "gstrtpmpadepay.h"
 #include "gstrtpmpvdepay.h"
-#include "gstrtpmpvpay.h"
 #include "gstrtph263pdepay.h"
 #include "gstrtph263ppay.h"
-#include "gstrtph263depay.h"
 #include "gstrtph263pay.h"
 #include "gstrtph264depay.h"
 #include "gstrtph264pay.h"
-#include "gstrtpjpegdepay.h"
-#include "gstrtpjpegpay.h"
 #include "gstrtpL16depay.h"
 #include "gstrtpL16pay.h"
 #include "gstasteriskh263.h"
-#include "gstrtpmp1sdepay.h"
 #include "gstrtpmp2tdepay.h"
 #include "gstrtpmp2tpay.h"
 #include "gstrtpmp4vdepay.h"
 #include "gstrtpmp4vpay.h"
 #include "gstrtpmp4adepay.h"
-#include "gstrtpmp4apay.h"
 #include "gstrtpmp4gdepay.h"
 #include "gstrtpmp4gpay.h"
 #include "gstrtpspeexpay.h"
@@ -70,8 +58,6 @@
 #include "gstrtptheorapay.h"
 #include "gstrtpvorbisdepay.h"
 #include "gstrtpvorbispay.h"
-#include "gstrtpvrawdepay.h"
-#include "gstrtpvrawpay.h"
 
 static gboolean
 plugin_init (GstPlugin * plugin)
@@ -82,30 +68,12 @@ plugin_init (GstPlugin * plugin)
   if (!gst_rtp_ac3_depay_plugin_init (plugin))
     return FALSE;
 
-  if (!gst_rtp_dv_depay_plugin_init (plugin))
-    return FALSE;
-
-  if (!gst_rtp_dv_pay_plugin_init (plugin))
-    return FALSE;
-
   if (!gst_rtp_ilbc_pay_plugin_init (plugin))
     return FALSE;
 
   if (!gst_rtp_ilbc_depay_plugin_init (plugin))
     return FALSE;
 
-  if (!gst_rtp_g726_depay_plugin_init (plugin))
-    return FALSE;
-
-  if (!gst_rtp_g726_pay_plugin_init (plugin))
-    return FALSE;
-
-  if (!gst_rtp_g729_depay_plugin_init (plugin))
-    return FALSE;
-
-  if (!gst_rtp_g729_pay_plugin_init (plugin))
-    return FALSE;
-
   if (!gst_rtp_gsm_depay_plugin_init (plugin))
     return FALSE;
 
@@ -139,18 +107,12 @@ plugin_init (GstPlugin * plugin)
   if (!gst_rtp_mpv_depay_plugin_init (plugin))
     return FALSE;
 
-  if (!gst_rtp_mpv_pay_plugin_init (plugin))
-    return FALSE;
-
   if (!gst_rtp_h263p_pay_plugin_init (plugin))
     return FALSE;
 
   if (!gst_rtp_h263p_depay_plugin_init (plugin))
     return FALSE;
 
-  if (!gst_rtp_h263_depay_plugin_init (plugin))
-    return FALSE;
-
   if (!gst_rtp_h263_pay_plugin_init (plugin))
     return FALSE;
 
@@ -160,12 +122,6 @@ plugin_init (GstPlugin * plugin)
   if (!gst_rtp_h264_pay_plugin_init (plugin))
     return FALSE;
 
-  if (!gst_rtp_jpeg_depay_plugin_init (plugin))
-    return FALSE;
-
-  if (!gst_rtp_jpeg_pay_plugin_init (plugin))
-    return FALSE;
-
   if (!gst_rtp_L16_pay_plugin_init (plugin))
     return FALSE;
 
@@ -175,9 +131,6 @@ plugin_init (GstPlugin * plugin)
   if (!gst_asteriskh263_plugin_init (plugin))
     return FALSE;
 
-  if (!gst_rtp_mp1s_depay_plugin_init (plugin))
-    return FALSE;
-
   if (!gst_rtp_mp2t_depay_plugin_init (plugin))
     return FALSE;
 
@@ -190,9 +143,6 @@ plugin_init (GstPlugin * plugin)
   if (!gst_rtp_mp4v_depay_plugin_init (plugin))
     return FALSE;
 
-  if (!gst_rtp_mp4a_pay_plugin_init (plugin))
-    return FALSE;
-
   if (!gst_rtp_mp4a_depay_plugin_init (plugin))
     return FALSE;
 
@@ -223,12 +173,6 @@ plugin_init (GstPlugin * plugin)
   if (!gst_rtp_vorbis_pay_plugin_init (plugin))
     return FALSE;
 
-  if (!gst_rtp_vraw_depay_plugin_init (plugin))
-    return FALSE;
-
-  if (!gst_rtp_vraw_pay_plugin_init (plugin))
-    return FALSE;
-
   return TRUE;
 }
 
