@@ -26,6 +26,9 @@
 #include <string.h>
 #include "gstrtpmpadepay.h"
 
+GST_DEBUG_CATEGORY_STATIC (rtpmpadepay_debug);
+#define GST_CAT_DEFAULT (rtpmpadepay_debug)
+
 /* elementfactory information */
 static const GstElementDetails gst_rtp_mpadepay_details =
 GST_ELEMENT_DETAILS ("RTP packet depayloader",
@@ -43,7 +46,6 @@ enum
 enum
 {
   ARG_0,
-  ARG_FREQUENCY
 };
 
 static GstStaticPadTemplate gst_rtp_mpa_depay_src_template =
@@ -54,20 +56,26 @@ GST_STATIC_PAD_TEMPLATE ("src",
     );
 
 static GstStaticPadTemplate gst_rtp_mpa_depay_sink_template =
-GST_STATIC_PAD_TEMPLATE ("sink",
+    GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_SINK,
     GST_PAD_ALWAYS,
     GST_STATIC_CAPS ("application/x-rtp, "
         "media = (string) \"audio\", "
-        "clock-rate = (int) 90000, " "encoding-name = (string) \"MPA\"")
+        "payload = (int) " GST_RTP_PAYLOAD_DYNAMIC_STRING ", "
+        "clock-rate = (int) 90000, " "encoding-name = (string) \"MPA\";"
+        "application/x-rtp, "
+        "media = (string) \"audio\", "
+        "payload = (int) " GST_RTP_PAYLOAD_MPA_STRING ", "
+        "clock-rate = (int) 90000")
     );
 
+GST_BOILERPLATE (GstRtpMPADepay, gst_rtp_mpa_depay, GstBaseRTPDepayload,
+    GST_TYPE_BASE_RTP_DEPAYLOAD);
 
-static void gst_rtp_mpa_depay_class_init (GstRtpMPADepayClass * klass);
-static void gst_rtp_mpa_depay_base_init (GstRtpMPADepayClass * klass);
-static void gst_rtp_mpa_depay_init (GstRtpMPADepay * rtpmpadepay);
-
-static GstFlowReturn gst_rtp_mpa_depay_chain (GstPad * pad, GstBuffer * buffer);
+static gboolean gst_rtp_mpa_depay_setcaps (GstBaseRTPDepayload * depayload,
+    GstCaps * caps);
+static GstBuffer *gst_rtp_mpa_depay_process (GstBaseRTPDepayload * depayload,
+    GstBuffer * buf);
 
 static void gst_rtp_mpa_depay_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec);
@@ -77,35 +85,8 @@ static void gst_rtp_mpa_depay_get_property (GObject * object, guint prop_id,
 static GstStateChangeReturn gst_rtp_mpa_depay_change_state (GstElement *
     element, GstStateChange transition);
 
-static GstElementClass *parent_class = NULL;
-
-static GType
-gst_rtp_mpa_depay_get_type (void)
-{
-  static GType rtpmpadepay_type = 0;
-
-  if (!rtpmpadepay_type) {
-    static const GTypeInfo rtpmpadepay_info = {
-      sizeof (GstRtpMPADepayClass),
-      (GBaseInitFunc) gst_rtp_mpa_depay_base_init,
-      NULL,
-      (GClassInitFunc) gst_rtp_mpa_depay_class_init,
-      NULL,
-      NULL,
-      sizeof (GstRtpMPADepay),
-      0,
-      (GInstanceInitFunc) gst_rtp_mpa_depay_init,
-    };
-
-    rtpmpadepay_type =
-        g_type_register_static (GST_TYPE_ELEMENT, "GstRtpMPADepay",
-        &rtpmpadepay_info, 0);
-  }
-  return rtpmpadepay_type;
-}
-
 static void
-gst_rtp_mpa_depay_base_init (GstRtpMPADepayClass * klass)
+gst_rtp_mpa_depay_base_init (gpointer klass)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
 
@@ -122,9 +103,11 @@ gst_rtp_mpa_depay_class_init (GstRtpMPADepayClass * klass)
 {
   GObjectClass *gobject_class;
   GstElementClass *gstelement_class;
+  GstBaseRTPDepayloadClass *gstbasertpdepayload_class;
 
   gobject_class = (GObjectClass *) klass;
   gstelement_class = (GstElementClass *) klass;
+  gstbasertpdepayload_class = (GstBaseRTPDepayloadClass *) klass;
 
   parent_class = g_type_class_peek_parent (klass);
 
@@ -132,44 +115,52 @@ gst_rtp_mpa_depay_class_init (GstRtpMPADepayClass * klass)
   gobject_class->get_property = gst_rtp_mpa_depay_get_property;
 
   gstelement_class->change_state = gst_rtp_mpa_depay_change_state;
+
+  gstbasertpdepayload_class->set_caps = gst_rtp_mpa_depay_setcaps;
+  gstbasertpdepayload_class->process = gst_rtp_mpa_depay_process;
+
+  GST_DEBUG_CATEGORY_INIT (rtpmpadepay_debug, "rtpmpadepay", 0,
+      "MPEG Audio RTP Depayloader");
 }
 
 static void
-gst_rtp_mpa_depay_init (GstRtpMPADepay * rtpmpadepay)
+gst_rtp_mpa_depay_init (GstRtpMPADepay * rtpmpadepay,
+    GstRtpMPADepayClass * klass)
 {
-  rtpmpadepay->srcpad =
-      gst_pad_new_from_static_template (&gst_rtp_mpa_depay_src_template, "src");
-  gst_element_add_pad (GST_ELEMENT (rtpmpadepay), rtpmpadepay->srcpad);
-
-  rtpmpadepay->sinkpad =
-      gst_pad_new_from_static_template (&gst_rtp_mpa_depay_sink_template,
-      "sink");
-  gst_pad_set_chain_function (rtpmpadepay->sinkpad, gst_rtp_mpa_depay_chain);
-  gst_element_add_pad (GST_ELEMENT (rtpmpadepay), rtpmpadepay->sinkpad);
 }
 
-static GstFlowReturn
-gst_rtp_mpa_depay_chain (GstPad * pad, GstBuffer * buf)
+static gboolean
+gst_rtp_mpa_depay_setcaps (GstBaseRTPDepayload * depayload, GstCaps * caps)
+{
+  GstStructure *structure;
+  GstRtpMPADepay *rtpmpadepay;
+  gint clock_rate = 90000;      /* default */
+
+  rtpmpadepay = GST_RTP_MPA_DEPAY (depayload);
+
+  structure = gst_caps_get_structure (caps, 0);
+
+  gst_structure_get_int (structure, "clock-rate", &clock_rate);
+  depayload->clock_rate = clock_rate;
+
+  return TRUE;
+}
+
+static GstBuffer *
+gst_rtp_mpa_depay_process (GstBaseRTPDepayload * depayload, GstBuffer * buf)
 {
   GstRtpMPADepay *rtpmpadepay;
   GstBuffer *outbuf;
-  guint8 pt;
-  GstFlowReturn ret;
 
-  rtpmpadepay = GST_RTP_MPA_DEPAY (GST_OBJECT_PARENT (pad));
+  rtpmpadepay = GST_RTP_MPA_DEPAY (depayload);
 
   if (!gst_rtp_buffer_validate (buf))
     goto bad_packet;
 
-  if ((pt = gst_rtp_buffer_get_payload_type (buf)) != GST_RTP_PAYLOAD_MPA)
-    goto bad_payload;
-
-
   {
     gint payload_len;
     guint8 *payload;
     guint16 frag_offset;
-    guint32 timestamp;
 
     payload_len = gst_rtp_buffer_get_payload_len (buf);
     payload = gst_rtp_buffer_get_payload (buf);
@@ -187,47 +178,39 @@ gst_rtp_mpa_depay_chain (GstPad * pad, GstBuffer * buf)
      */
     frag_offset = (payload[2] << 8) | payload[3];
 
-    payload_len -= 4;
-    payload += 4;
-
-    timestamp = gst_rtp_buffer_get_timestamp (buf);
-
-    outbuf = gst_buffer_new_and_alloc (payload_len);
-
-    //GST_BUFFER_TIMESTAMP (outbuf) = timestamp * GST_SECOND / 90000;
-
-    memcpy (GST_BUFFER_DATA (outbuf), payload, payload_len);
+    /* subbuffer skipping the 4 header bytes */
+    outbuf = gst_rtp_buffer_get_payload_subbuffer (buf, 4, -1);
 
     GST_DEBUG_OBJECT (rtpmpadepay,
         "gst_rtp_mpa_depay_chain: pushing buffer of size %d",
         GST_BUFFER_SIZE (outbuf));
 
-    gst_buffer_unref (buf);
-
     /* FIXME, we can push half mpeg frames when they are split over multiple
      * RTP packets */
-    ret = gst_pad_push (rtpmpadepay->srcpad, outbuf);
+    return outbuf;
   }
 
-  return ret;
+  return NULL;
 
 bad_packet:
   {
-    GST_DEBUG_OBJECT (rtpmpadepay, "Packet did not validate");
-    gst_buffer_unref (buf);
-    return GST_FLOW_ERROR;
+    GST_ELEMENT_WARNING (rtpmpadepay, STREAM, DECODE,
+        ("Packet did not validate."), (NULL));
+    return NULL;
   }
+#if 0
 bad_payload:
   {
-    GST_DEBUG_OBJECT (rtpmpadepay, "Unexpected payload type %u", pt);
-    gst_buffer_unref (buf);
-    return GST_FLOW_ERROR;
+    GST_ELEMENT_WARNING (rtpmpadepay, STREAM, DECODE,
+        ("Unexpected payload type."), (NULL));
+    return NULL;
   }
+#endif
 empty_packet:
   {
-    GST_DEBUG_OBJECT (rtpmpadepay, "Empty payload");
-    gst_buffer_unref (buf);
-    return GST_FLOW_OK;
+    GST_ELEMENT_WARNING (rtpmpadepay, STREAM, DECODE,
+        ("Empty Payload."), (NULL));
+    return NULL;
   }
 }
 
