@@ -1,5 +1,5 @@
 /* GStreamer
- * Copyright (C) <2005> Wim Taymans <wim@fluendo.com>
+ * Copyright (C) <2005,2006> Wim Taymans <wim@fluendo.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -16,12 +16,38 @@
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  */
+/*
+ * Unless otherwise indicated, Source Code is licensed under MIT license.
+ * See further explanation attached in License Statement (distributed in the file
+ * LICENSE).
+ *
+ * Permission is hereby granted, free of charge, to any person obtaining a copy of
+ * this software and associated documentation files (the "Software"), to deal in
+ * the Software without restriction, including without limitation the rights to
+ * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
+ * of the Software, and to permit persons to whom the Software is furnished to do
+ * so, subject to the following conditions:
+ *
+ * The above copyright notice and this permission notice shall be included in all
+ * copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+ * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+ * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+ * SOFTWARE.
+ */
 
 #include <stdlib.h>
 #include <string.h>
 
 #include "sdpmessage.h"
 
+/* FIXME, is currently allocated on the stack */
+#define MAX_LINE_LEN	1024 * 16
+
 #define FREE_STRING(field)      g_free ((field)); (field) = NULL;
 #define FREE_ARRAY(field)       \
 G_STMT_START {                  \
@@ -31,11 +57,15 @@ G_STMT_START {                  \
 } G_STMT_END
 #define REPLACE_STRING(field,val)       FREE_STRING(field);field=g_strdup (val);
 
-#define INIT_ARRAY(field,type)  \
-G_STMT_START {                  \
-  if (field)                    \
-    g_array_set_size (field,0); \
-  else                          \
+#define INIT_ARRAY(field,type,init_func)  		\
+G_STMT_START {                   			\
+  if (field) {                   			\
+    guint i;			 			\
+    for(i=0; i<field->len; i++)				\
+      init_func (&g_array_index(field, type, i));	\
+    g_array_set_size (field,0); 			\
+  }				 			\
+  else                          			\
     field = g_array_new (FALSE, TRUE, sizeof(type));    \
 } G_STMT_END
 
@@ -54,30 +84,92 @@ char* sdp_message_get_##field (SDPMessage *msg) {                       \
 gint sdp_message_##field##_len (SDPMessage *msg) {                      \
   return ((msg)->field->len);                                           \
 }
-#define DEFINE_ARRAY_GETTER(method,field,type)                                  \
-type sdp_message_get_##method (SDPMessage *msg, gint i) {               \
-  return g_array_index ((msg)->field, type, i);                         \
+#define DEFINE_ARRAY_GETTER(method,field,type)                          \
+type sdp_message_get_##method (SDPMessage *msg, guint idx) {            \
+  return g_array_index ((msg)->field, type, idx);                       \
 }
-#define DEFINE_ARRAY_P_GETTER(method,field,type)                                        \
-type * sdp_message_get_##method (SDPMessage *msg, gint i) {             \
-  return &g_array_index ((msg)->field, type, i);                                \
+#define DEFINE_ARRAY_P_GETTER(method,field,type)                        \
+type * sdp_message_get_##method (SDPMessage *msg, guint idx) {          \
+  return &g_array_index ((msg)->field, type, idx);                      \
 }
-#define DEFINE_ARRAY_ADDER(method,field,type,dup_method)                        \
+#define DEFINE_ARRAY_ADDER(method,field,type,dup_method)                \
 RTSPResult sdp_message_add_##method (SDPMessage *msg, type val) {       \
   type v = dup_method(val);                                             \
   g_array_append_val((msg)->field, v);                                  \
   return RTSP_OK;                                                       \
 }
 
+static void
+sdp_origin_init (SDPOrigin * origin)
+{
+  FREE_STRING (origin->username);
+  FREE_STRING (origin->sess_id);
+  FREE_STRING (origin->sess_version);
+  FREE_STRING (origin->nettype);
+  FREE_STRING (origin->addrtype);
+  FREE_STRING (origin->addr);
+}
 
+static void
+sdp_connection_init (SDPConnection * connection)
+{
+  FREE_STRING (connection->nettype);
+  FREE_STRING (connection->addrtype);
+  FREE_STRING (connection->address);
+  connection->ttl = 0;
+  connection->addr_number = 0;
+}
 
+static void
+sdp_bandwidth_init (SDPBandwidth * bandwidth)
+{
+  FREE_STRING (bandwidth->bwtype);
+  bandwidth->bandwidth = 0;
+}
+
+static void
+sdp_time_init (SDPTime * time)
+{
+  FREE_STRING (time->start);
+  FREE_STRING (time->stop);
+  time->n_repeat = 0;
+}
+
+static void
+sdp_zone_init (SDPZone * zone)
+{
+  FREE_STRING (zone->time);
+  FREE_STRING (zone->typed_time);
+}
+
+static void
+sdp_key_init (SDPKey * key)
+{
+  FREE_STRING (key->type);
+  FREE_STRING (key->data);
+}
+
+static void
+sdp_attribute_init (SDPAttribute * attr)
+{
+  FREE_STRING (attr->key);
+  FREE_STRING (attr->value);
+}
+
+/**
+ * sdp_message_new:
+ * @msg: pointer to new #SDPMessage
+ *
+ * Allocate a new SDPMessage and store the result in @msg.
+ *
+ * Returns: a #RTSPResult.
+ */
 RTSPResult
 sdp_message_new (SDPMessage ** msg)
 {
   SDPMessage *newmsg;
 
-  if (msg == NULL)
-    return RTSP_EINVAL;
+  g_return_val_if_fail (msg != NULL, RTSP_EINVAL);
 
   newmsg = g_new0 (SDPMessage, 1);
 
@@ -86,45 +178,58 @@ sdp_message_new (SDPMessage ** msg)
   return sdp_message_init (newmsg);
 }
 
+/**
+ * sdp_message_init:
+ * @msg: an #SDPMessage
+ *
+ * Initialize @msg so that its contents are as if it was freshly allocated
+ * with sdp_message_new(). This function is mostly used to initialize a message
+ * allocated on the stack. sdp_message_uninit() undoes this operation.
+ *
+ * When this function is invoked on newly allocated data (with malloc or on the
+ * stack), its contents should be set to 0 before calling this function.
+ *
+ * Returns: a #RTSPResult.
+ */
 RTSPResult
 sdp_message_init (SDPMessage * msg)
 {
-  if (msg == NULL)
-    return RTSP_EINVAL;
+  g_return_val_if_fail (msg != NULL, RTSP_EINVAL);
 
   FREE_STRING (msg->version);
-  FREE_STRING (msg->origin.username);
-  FREE_STRING (msg->origin.sess_id);
-  FREE_STRING (msg->origin.sess_version);
-  FREE_STRING (msg->origin.nettype);
-  FREE_STRING (msg->origin.addrtype);
-  FREE_STRING (msg->origin.addr);
+  sdp_origin_init (&msg->origin);
   FREE_STRING (msg->session_name);
   FREE_STRING (msg->information);
   FREE_STRING (msg->uri);
-  INIT_ARRAY (msg->emails, gchar *);
-  INIT_ARRAY (msg->phones, gchar *);
-  FREE_STRING (msg->connection.nettype);
-  FREE_STRING (msg->connection.addrtype);
-  FREE_STRING (msg->connection.address);
-  msg->connection.ttl = 0;
-  msg->connection.addr_number = 0;
-  INIT_ARRAY (msg->bandwidths, SDPBandwidth);
-  INIT_ARRAY (msg->times, SDPTime);
-  INIT_ARRAY (msg->zones, SDPZone);
-  FREE_STRING (msg->key.type);
-  FREE_STRING (msg->key.data);
-  INIT_ARRAY (msg->attributes, SDPAttribute);
-  INIT_ARRAY (msg->medias, SDPMedia);
+  INIT_ARRAY (msg->emails, gchar *, g_free);
+  INIT_ARRAY (msg->phones, gchar *, g_free);
+  sdp_connection_init (&msg->connection);
+  INIT_ARRAY (msg->bandwidths, SDPBandwidth, sdp_bandwidth_init);
+  INIT_ARRAY (msg->times, SDPTime, sdp_time_init);
+  INIT_ARRAY (msg->zones, SDPZone, sdp_zone_init);
+  sdp_key_init (&msg->key);
+  INIT_ARRAY (msg->attributes, SDPAttribute, sdp_attribute_init);
+  INIT_ARRAY (msg->medias, SDPMedia, sdp_media_uninit);
 
   return RTSP_OK;
 }
 
+/**
+ * sdp_message_uninit:
+ * @msg: an #SDPMessage
+ *
+ * Free all resources allocated in @msg. @msg should not be used anymore after
+ * this function. This function should be used when @msg was allocated on the
+ * stack and initialized with sdp_message_init().
+ *
+ * Returns: a #RTSPResult.
+ */
 RTSPResult
-sdp_message_clean (SDPMessage * msg)
+sdp_message_uninit (SDPMessage * msg)
 {
-  if (msg == NULL)
-    return RTSP_EINVAL;
+  g_return_val_if_fail (msg != NULL, RTSP_EINVAL);
+
+  sdp_message_init (msg);
 
   FREE_ARRAY (msg->emails);
   FREE_ARRAY (msg->phones);
@@ -137,27 +242,41 @@ sdp_message_clean (SDPMessage * msg)
   return RTSP_OK;
 }
 
+/**
+ * sdp_message_free:
+ * @msg: an #SDPMessage
+ *
+ * Free all resources allocated by @msg. @msg should not be used anymore after
+ * this function. This function should be used when @msg was dynamically
+ * allocated with sdp_message_new().
+ *
+ * Returns: a #RTSPResult.
+ */
 RTSPResult
 sdp_message_free (SDPMessage * msg)
 {
-  if (msg == NULL)
-    return RTSP_EINVAL;
-
-  sdp_message_clean (msg);
+  g_return_val_if_fail (msg != NULL, RTSP_EINVAL);
 
+  sdp_message_uninit (msg);
   g_free (msg);
 
   return RTSP_OK;
 }
 
-
+/**
+ * sdp_media_new:
+ * @media: pointer to new #SDPMedia
+ *
+ * Allocate a new SDPMedia and store the result in @media.
+ *
+ * Returns: a #RTSPResult.
+ */
 RTSPResult
 sdp_media_new (SDPMedia ** media)
 {
   SDPMedia *newmedia;
 
-  if (media == NULL)
-    return RTSP_EINVAL;
+  g_return_val_if_fail (media != NULL, RTSP_EINVAL);
 
   newmedia = g_new0 (SDPMedia, 1);
 
@@ -166,23 +285,79 @@ sdp_media_new (SDPMedia ** media)
   return sdp_media_init (newmedia);
 }
 
+/**
+ * sdp_media_init:
+ * @media: a #SDPMedia
+ *
+ * Initialize @media so that its contents are as if it was freshly allocated
+ * with sdp_media_new(). This function is mostly used to initialize a media
+ * allocated on the stack. sdp_media_uninit() undoes this operation.
+ *
+ * When this function is invoked on newly allocated data (with malloc or on the
+ * stack), its contents should be set to 0 before calling this function.
+ *
+ * Returns: a #RTSPResult.
+ */
 RTSPResult
 sdp_media_init (SDPMedia * media)
 {
-  if (media == NULL)
-    return RTSP_EINVAL;
+  g_return_val_if_fail (media != NULL, RTSP_EINVAL);
 
   FREE_STRING (media->media);
   media->port = 0;
   media->num_ports = 0;
   FREE_STRING (media->proto);
-  INIT_ARRAY (media->fmts, gchar *);
+  INIT_ARRAY (media->fmts, gchar *, g_free);
   FREE_STRING (media->information);
-  INIT_ARRAY (media->connections, SDPConnection);
-  INIT_ARRAY (media->bandwidths, SDPBandwidth);
-  FREE_STRING (media->key.type);
-  FREE_STRING (media->key.data);
-  INIT_ARRAY (media->attributes, SDPAttribute);
+  INIT_ARRAY (media->connections, SDPConnection, sdp_connection_init);
+  INIT_ARRAY (media->bandwidths, SDPBandwidth, sdp_bandwidth_init);
+  sdp_key_init (&media->key);
+  INIT_ARRAY (media->attributes, SDPAttribute, sdp_attribute_init);
+
+  return RTSP_OK;
+}
+
+/**
+ * sdp_media_uninit:
+ * @media: an #SDPMedia
+ *
+ * Free all resources allocated in @media. @media should not be used anymore after
+ * this function. This function should be used when @media was allocated on the
+ * stack and initialized with sdp_media_init().
+ *
+ * Returns: a #RTSPResult.
+ */
+RTSPResult
+sdp_media_uninit (SDPMedia * media)
+{
+  g_return_val_if_fail (media != NULL, RTSP_EINVAL);
+
+  sdp_media_init (media);
+  FREE_ARRAY (media->fmts);
+  FREE_ARRAY (media->connections);
+  FREE_ARRAY (media->bandwidths);
+  FREE_ARRAY (media->attributes);
+
+  return RTSP_OK;
+}
+
+/**
+ * sdp_media_free:
+ * @media: an #SDPMedia
+ *
+ * Free all resources allocated by @media. @media should not be used anymore after
+ * this function. This function should be used when @media was dynamically
+ * allocated with sdp_media_new().
+ *
+ * Returns: a #RTSPResult.
+ */
+RTSPResult
+sdp_media_free (SDPMedia * media)
+{
+  g_return_val_if_fail (media != NULL, RTSP_EINVAL);
+
+  sdp_media_uninit (media);
+  g_free (media);
 
   return RTSP_OK;
 }
@@ -300,20 +475,30 @@ sdp_message_get_key (SDPMessage * msg)
 DEFINE_ARRAY_LEN (attributes);
 DEFINE_ARRAY_P_GETTER (attribute, attributes, SDPAttribute);
 gchar *
-sdp_message_get_attribute_val (SDPMessage * msg, gchar * key)
+sdp_message_get_attribute_val_n (SDPMessage * msg, gchar * key, guint nth)
 {
-  gint i;
+  guint i;
 
   for (i = 0; i < msg->attributes->len; i++) {
     SDPAttribute *attr;
 
     attr = &g_array_index (msg->attributes, SDPAttribute, i);
-    if (!strcmp (attr->key, key))
-      return attr->value;
+    if (!strcmp (attr->key, key)) {
+      if (nth == 0)
+        return attr->value;
+      else
+        nth--;
+    }
   }
   return NULL;
 }
 
+gchar *
+sdp_message_get_attribute_val (SDPMessage * msg, gchar * key)
+{
+  return sdp_message_get_attribute_val_n (msg, key, 0);
+}
+
 RTSPResult
 sdp_message_add_attribute (SDPMessage * msg, gchar * key, gchar * value)
 {
@@ -329,6 +514,18 @@ sdp_message_add_attribute (SDPMessage * msg, gchar * key, gchar * value)
 
 DEFINE_ARRAY_LEN (medias);
 DEFINE_ARRAY_P_GETTER (media, medias, SDPMedia);
+
+/**
+ * sdp_message_add_media:
+ * @msg: an #SDPMessage
+ * @media: an #SDPMedia to add
+ *
+ * Adds @media to the array of medias in @msg. This function takes ownership of
+ * the contents of @media so that @media will have to be reinitialized with
+ * gst_media_init() before it can be used again.
+ *
+ * Returns: an #RTSPResult.
+ */
 RTSPResult
 sdp_message_add_media (SDPMessage * msg, SDPMedia * media)
 {
@@ -340,6 +537,7 @@ sdp_message_add_media (SDPMessage * msg, SDPMedia * media)
   nmedia = &g_array_index (msg->medias, SDPMedia, len);
 
   memcpy (nmedia, media, sizeof (SDPMedia));
+  memset (media, 0, sizeof (SDPMedia));
 
   return RTSP_OK;
 }
@@ -384,33 +582,49 @@ sdp_media_add_format (SDPMedia * media, gchar * format)
   return RTSP_OK;
 }
 
-gchar *
-sdp_media_get_attribute_val (SDPMedia * media, gchar * key)
+SDPAttribute *
+sdp_media_get_attribute (SDPMedia * media, guint idx)
 {
-  gint i;
+  return &g_array_index (media->attributes, SDPAttribute, idx);
+}
+
+gchar *
+sdp_media_get_attribute_val_n (SDPMedia * media, gchar * key, guint nth)
+{
+  guint i;
 
   for (i = 0; i < media->attributes->len; i++) {
     SDPAttribute *attr;
 
     attr = &g_array_index (media->attributes, SDPAttribute, i);
-    if (!strcmp (attr->key, key))
-      return attr->value;
+    if (!strcmp (attr->key, key)) {
+      if (nth == 0)
+        return attr->value;
+      else
+        nth--;
+    }
   }
   return NULL;
 }
 
 gchar *
-sdp_media_get_format (SDPMedia * media, gint i)
+sdp_media_get_attribute_val (SDPMedia * media, gchar * key)
 {
-  if (i >= media->fmts->len)
+  return sdp_media_get_attribute_val_n (media, key, 0);
+}
+
+gchar *
+sdp_media_get_format (SDPMedia * media, guint idx)
+{
+  if (idx >= media->fmts->len)
     return NULL;
-  return g_array_index (media->fmts, gchar *, i);
+  return g_array_index (media->fmts, gchar *, idx);
 }
 
 static void
-read_string (gchar * dest, gint size, gchar ** src)
+read_string (gchar * dest, guint size, gchar ** src)
 {
-  gint idx;
+  guint idx;
 
   idx = 0;
   /* skip spaces */
@@ -427,9 +641,9 @@ read_string (gchar * dest, gint size, gchar ** src)
 }
 
 static void
-read_string_del (gchar * dest, gint size, gchar del, gchar ** src)
+read_string_del (gchar * dest, guint size, gchar del, gchar ** src)
 {
-  gint idx;
+  guint idx;
 
   idx = 0;
   /* skip spaces */
@@ -461,7 +675,7 @@ typedef struct
 static gboolean
 sdp_parse_line (SDPContext * c, gchar type, gchar * buffer)
 {
-  gchar str[4096];
+  gchar str[8192];
   gchar *p = buffer;
 
 #define READ_STRING(field) read_string (str, sizeof(str), &p);REPLACE_STRING (field, str);
@@ -509,7 +723,7 @@ sdp_parse_line (SDPContext * c, gchar type, gchar * buffer)
       break;
     case 'b':
     {
-      gchar str2[4096];
+      gchar str2[MAX_LINE_LEN];
 
       read_string_del (str, sizeof (str), ':', &p);
       read_string (str2, sizeof (str2), &p);
@@ -526,7 +740,7 @@ sdp_parse_line (SDPContext * c, gchar type, gchar * buffer)
       break;
     case 'a':
       read_string_del (str, sizeof (str), ':', &p);
-      if (p != '\0')
+      if (*p != '\0')
         p++;
       if (c->state == SDP_SESSION)
         sdp_message_add_attribute (c->msg, str, p);
@@ -536,9 +750,10 @@ sdp_parse_line (SDPContext * c, gchar type, gchar * buffer)
     case 'm':
     {
       gchar *slash;
-      SDPMedia nmedia = { 0 };
+      SDPMedia nmedia;
 
       c->state = SDP_MEDIA;
+      memset (&nmedia, 0, sizeof (nmedia));
       sdp_media_init (&nmedia);
 
       READ_STRING (nmedia.media);
@@ -575,11 +790,12 @@ sdp_message_parse_buffer (guint8 * data, guint size, SDPMessage * msg)
   gchar *p;
   SDPContext c;
   gchar type;
-  gchar buffer[4096];
-  gint idx = 0;
+  gchar buffer[MAX_LINE_LEN];
+  guint idx = 0;
 
-  if (msg == NULL || data == NULL || size == 0)
-    return RTSP_EINVAL;
+  g_return_val_if_fail (msg != NULL, RTSP_EINVAL);
+  g_return_val_if_fail (data != NULL, RTSP_EINVAL);
+  g_return_val_if_fail (size != 0, RTSP_EINVAL);
 
   c.state = SDP_SESSION;
   c.msg = msg;
@@ -625,7 +841,7 @@ print_media (SDPMedia * media)
   g_print ("   num_ports:   '%d'\n", media->num_ports);
   g_print ("   proto:       '%s'\n", media->proto);
   if (media->fmts->len > 0) {
-    gint i;
+    guint i;
 
     g_print ("   formats:\n");
     for (i = 0; i < media->fmts->len; i++) {
@@ -637,7 +853,7 @@ print_media (SDPMedia * media)
   g_print ("    type:       '%s'\n", media->key.type);
   g_print ("    data:       '%s'\n", media->key.data);
   if (media->attributes->len > 0) {
-    gint i;
+    guint i;
 
     g_print ("   attributes:\n");
     for (i = 0; i < media->attributes->len; i++) {
@@ -651,8 +867,7 @@ print_media (SDPMedia * media)
 RTSPResult
 sdp_message_dump (SDPMessage * msg)
 {
-  if (msg == NULL)
-    return RTSP_EINVAL;
+  g_return_val_if_fail (msg != NULL, RTSP_EINVAL);
 
   g_print ("sdp packet %p:\n", msg);
   g_print (" version:       '%s'\n", msg->version);
@@ -666,8 +881,9 @@ sdp_message_dump (SDPMessage * msg)
   g_print (" session_name:  '%s'\n", msg->session_name);
   g_print (" information:   '%s'\n", msg->information);
   g_print (" uri:           '%s'\n", msg->uri);
+
   if (msg->emails->len > 0) {
-    gint i;
+    guint i;
 
     g_print (" emails:\n");
     for (i = 0; i < msg->emails->len; i++) {
@@ -675,7 +891,7 @@ sdp_message_dump (SDPMessage * msg)
     }
   }
   if (msg->phones->len > 0) {
-    gint i;
+    guint i;
 
     g_print (" phones:\n");
     for (i = 0; i < msg->phones->len; i++) {
@@ -692,7 +908,7 @@ sdp_message_dump (SDPMessage * msg)
   g_print ("  type:         '%s'\n", msg->key.type);
   g_print ("  data:         '%s'\n", msg->key.data);
   if (msg->attributes->len > 0) {
-    gint i;
+    guint i;
 
     g_print (" attributes:\n");
     for (i = 0; i < msg->attributes->len; i++) {
@@ -702,7 +918,7 @@ sdp_message_dump (SDPMessage * msg)
     }
   }
   if (msg->medias->len > 0) {
-    gint i;
+    guint i;
 
     g_print (" medias:\n");
     for (i = 0; i < msg->medias->len; i++) {
@@ -710,7 +926,5 @@ sdp_message_dump (SDPMessage * msg)
       print_media (&g_array_index (msg->medias, SDPMedia, i));
     }
   }
-
-
   return RTSP_OK;
 }
