@@ -21,23 +21,32 @@
 
 /**
  * SECTION:element-waveformsink
+ * @short_description: output sound using WaveForm API
  *
- * This element lets you output sound using the Windows WaveForm API.
- *
+ * <refsect2>
+ * <para>
+ * This element lets you output sound using the WaveForm API.
+ * </para>
+ * <para>
  * Note that you should almost always use generic audio conversion elements
  * like audioconvert and audioresample in front of an audiosink to make sure
  * your pipeline works under all circumstances (those conversion elements will
  * act in passthrough-mode if no conversion is necessary).
- *
- * <refsect2>
+ * </para>
  * <title>Example pipelines</title>
- * |[
- * gst-launch -v audiotestsrc ! audioconvert ! volume volume=0.1 ! waveformsink
- * ]| will output a sine wave (continuous beep sound) to your sound card (with
+ * <para>
+ * <programlisting>
+ * gst-launch-0.10 -v audiotestsrc ! audioconvert ! volume volume=0.1 ! waveformsink
+ * </programlisting>
+ * will output a sine wave (continuous beep sound) to your sound card (with
  * a very low volume as precaution).
- * |[
- * gst-launch -v filesrc location=music.ogg ! decodebin ! audioconvert ! audioresample ! waveformsink
- * ]| will play an Ogg/Vorbis audio file and output it.
+ * </para>
+ * <para>
+ * <programlisting>
+ * gst-launch-0.10 -v filesrc location=music.ogg ! decodebin ! audioconvert ! audioresample ! waveformsink
+ * </programlisting>
+ * will play an Ogg/Vorbis audio file and output it.
+ * </para>
  * </refsect2>
  */
 
@@ -49,6 +58,13 @@
 
 GST_DEBUG_CATEGORY_STATIC (waveformsink_debug);
 
+/* elementfactory information */
+static const GstElementDetails gst_waveform_sink_details =
+GST_ELEMENT_DETAILS ("WaveForm Audio Sink",
+    "Sink/Audio",
+    "Output to a sound card via WaveForm API",
+    "Sebastien Moutte <sebastien@moutte.net>");
+
 static void gst_waveform_sink_base_init (gpointer g_class);
 static void gst_waveform_sink_class_init (GstWaveFormSinkClass * klass);
 static void gst_waveform_sink_init (GstWaveFormSink * wfsink,
@@ -80,8 +96,8 @@ GstCaps *gst_waveform_sink_create_caps (gint rate, gint channels,
     gint bits_per_sample);
 WAVEHDR *bufferpool_get_buffer (GstWaveFormSink * wfsink, gpointer data,
     guint length);
-void CALLBACK waveOutProc (HWAVEOUT hwo, UINT uMsg, DWORD_PTR dwInstance,
-    DWORD_PTR dwParam1, DWORD_PTR dwParam2);
+void CALLBACK waveOutProc (HWAVEOUT hwo, UINT uMsg, unsigned long dwInstance,
+    DWORD dwParam1, DWORD dwParam2);
 
 static GstStaticPadTemplate waveformsink_sink_factory =
     GST_STATIC_PAD_TEMPLATE ("sink",
@@ -106,12 +122,9 @@ gst_waveform_sink_base_init (gpointer g_class)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
 
-  gst_element_class_set_details_simple (element_class, "WaveForm Audio Sink",
-      "Sink/Audio",
-      "Output to a sound card via WaveForm API",
-      "Sebastien Moutte <sebastien@moutte.net>");
-  gst_element_class_add_static_pad_template (element_class,
-      &waveformsink_sink_factory);
+  gst_element_class_set_details (element_class, &gst_waveform_sink_details);
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&waveformsink_sink_factory));
 }
 
 static void
@@ -131,9 +144,11 @@ gst_waveform_sink_class_init (GstWaveFormSinkClass * klass)
 
   parent_class = g_type_class_peek_parent (klass);
 
-  gobject_class->finalize = gst_waveform_sink_finalise;
-  gobject_class->get_property = gst_waveform_sink_get_property;
-  gobject_class->set_property = gst_waveform_sink_set_property;
+  gobject_class->finalize = GST_DEBUG_FUNCPTR (gst_waveform_sink_finalise);
+  gobject_class->get_property =
+      GST_DEBUG_FUNCPTR (gst_waveform_sink_get_property);
+  gobject_class->set_property =
+      GST_DEBUG_FUNCPTR (gst_waveform_sink_set_property);
 
   gstbasesink_class->get_caps = GST_DEBUG_FUNCPTR (gst_waveform_sink_getcaps);
 
@@ -378,7 +393,7 @@ gst_waveform_sink_prepare (GstAudioSink * asink, GstRingBufferSpec * spec)
 
   /* open the default audio device with the given caps */
   mmresult = waveOutOpen (&wfsink->hwaveout, WAVE_MAPPER,
-      &wfx, (DWORD_PTR) waveOutProc, (DWORD_PTR) wfsink, CALLBACK_FUNCTION);
+      &wfx, (DWORD) waveOutProc, (DWORD) wfsink, CALLBACK_FUNCTION);
   if (mmresult != MMSYSERR_NOERROR) {
     waveOutGetErrorText (mmresult, wfsink->error_string, ERROR_LENGTH - 1);
     GST_ELEMENT_ERROR (wfsink, RESOURCE, OPEN_WRITE,
@@ -420,7 +435,8 @@ gst_waveform_sink_unprepare (GstAudioSink * asink)
 
     for (index = 0; index < wfsink->buffer_count; index++) {
       if (wfsink->wave_buffers[index].dwFlags & WHDR_PREPARED) {
-        MMRESULT mmresult = waveOutUnprepareHeader (wfsink->hwaveout,
+        MMRESULT mmresult =
+            waveOutUnprepareHeader (wfsink->hwaveout,
             &wfsink->wave_buffers[index], sizeof (WAVEHDR));
         if (mmresult != MMSYSERR_NOERROR) {
           waveOutGetErrorText (mmresult, wfsink->error_string,
@@ -494,7 +510,7 @@ gst_waveform_sink_write (GstAudioSink * asink, gpointer data, guint length)
     memcpy (waveheader->lpData + waveheader->dwUser, data, bytes_to_write);
     waveheader->dwUser += bytes_to_write;
     remaining_length -= bytes_to_write;
-    data = (guint8 *) data + bytes_to_write;
+    data = (byte *) data + bytes_to_write;
 
     if (waveheader->dwUser == wfsink->buffer_size) {
       /* we have filled a buffer, let's prepare it and next write it to the device */
@@ -573,7 +589,7 @@ gst_waveform_sink_create_caps (gint rate, gint channels, gint bits_per_sample)
 
 void CALLBACK
 waveOutProc (HWAVEOUT hwo,
-    UINT uMsg, DWORD_PTR dwInstance, DWORD_PTR dwParam1, DWORD_PTR dwParam2)
+    UINT uMsg, unsigned long dwInstance, DWORD dwParam1, DWORD dwParam2)
 {
   GstWaveFormSink *wfsink = (GstWaveFormSink *) dwInstance;
 
